#
# Copyright (C) 2020  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.

"""A module providing downloaders of bank statements and possibly other resources."""

from abc import ABC, abstractclassmethod
from datetime import date
from urllib.parse import urljoin
from warnings import warn

import requests


class TellerDownloadError(Exception):
    """Error raised by Teller when statement download fails."""

    pass


class BankStatementDownloader(ABC):
    """Abstract class for statement downloaders."""

    def __init__(self, base_url: str, timeout: int):
        if not base_url.startswith('https://'):
            warn('Sensitive data is transferred over possibly insecure channel. Consider HTTPS.', UserWarning)
        self.base_url = base_url
        self.timeout = timeout

    @abstractclassmethod
    def get_statement(self, start_date: date, end_date: date) -> str:
        """Download content from a bank."""


class FioStatementDownloader(BankStatementDownloader):
    """Downloader for Fio bank statements."""

    date_format = '%Y-%m-%d'
    path_format = 'ib_api/rest/periods/{password}/{start}/{end}/transactions.xml'

    def __init__(self, base_url: str, password: str, timeout: int = 3):
        super().__init__(base_url, timeout)
        self.password = password

    def get_statement(self, start_date: date, end_date: date) -> str:
        """Download statement from Fio. DO NOT TO LOG EXCEPTIONS raised by this method. It may contain senstive data."""
        try:
            response = requests.get(self._compose_url(start_date, end_date), timeout=self.timeout)
            response.raise_for_status()
        except Exception:
            raise TellerDownloadError('Statement could not be retrieved.')
        return response.text

    def _compose_url(self, start_date: date, end_date: date) -> str:
        start = start_date.strftime(self.date_format)
        end = end_date.strftime(self.date_format)
        path = self.path_format.format(password=self.password, start=start, end=end)
        return urljoin(self.base_url, path)
