#
# Copyright (C) 2020  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.

"""A module providing parsers for bank statements."""
from abc import ABC, abstractmethod
from datetime import date
from io import StringIO
from pathlib import Path
from typing import Optional, TextIO, Union
from xml.etree.ElementTree import Element  # nosec - for typing only, for parsing use defusedxml, see below

from defusedxml.ElementTree import parse  # see: https://docs.python.org/3/library/xml.html#xml-vulnerabilities
from moneyed import Money

from teller.statement import BankStatement, Payment


class BankStatementParser(ABC):
    """Base class for Statement parsers."""

    @classmethod
    def parse_string(cls, source: str) -> BankStatement:
        """Parse Bank statement from a string - source is a string containing the statement to be parsed."""
        return cls.parse_file(StringIO(source))

    @classmethod
    @abstractmethod
    def parse_file(cls, source: Union[str, TextIO, Path]) -> BankStatement:
        """Parse Bank statement from a file - source is either a path to the file or an io object."""
        raise NotImplementedError('This is abstract base class use bank specific parser insted.')


class FioParser(BankStatementParser):
    """Parser for Fio bank statements."""

    date_format = '%Y-%m-%d%z'

    @classmethod
    def parse_file(cls, source: Union[str, Path, TextIO]) -> BankStatement:
        """Parse Bank statement from a file - source is either a path to the file or an io object."""
        # parse() can not handle Path type in python 3.5
        if isinstance(source, Path):
            source = str(source)

        tree = parse(source)
        header = tree.find('Info')

        if header is None:
            raise ValueError('Statement is missing header.')

        transaction_list = tree.find('TransactionList')

        account_number = FioParser._parse_account(header)
        statement = BankStatement(account_number=account_number)

        if transaction_list:
            for transaction in transaction_list:
                statement.add_payment(FioParser._parse_transaction(transaction))

        return statement

    @staticmethod
    def _parse_account(header: Element) -> str:
        account_id = header.findtext('accountId')
        bank_id = header.findtext('bankId')
        if account_id is not None and bank_id is not None:
            return '{}/{}'.format(account_id, bank_id)
        else:
            raise ValueError('Could not parse account number.')

    @staticmethod
    def _parse_transaction(element: Element) -> Payment:
        payment = Payment()

        payment.identifier = element.findtext('column_22', None)
        payment.transaction_date = FioParser._parse_date(element)
        payment.counter_account = FioParser._parse_counter_account(element)
        payment.name = element.findtext('column_10', None)
        payment.amount = FioParser._parse_amount(element)
        payment.description = element.findtext('column_16', None)
        payment.constant_symbol = element.findtext('column_4', None)
        payment.variable_symbol = element.findtext('column_5', None)
        payment.specific_symbol = element.findtext('column_6', None)

        return payment

    @staticmethod
    def _parse_amount(element: Element) -> Optional[Money]:
        amount = element.findtext('column_1', None)
        currency = element.findtext('column_14', None)
        if amount is not None and currency is not None:
            return Money(amount, currency)
        else:
            return None

    @staticmethod
    def _parse_date(element: Element) -> Optional[date]:
        date_str = element.findtext('column_0', None)
        if date_str:
            # the date is provided with timezone (e.g. 2012-12-21+01:00) which we ignore
            # also python 3.5 can not parse date
            year = int(date_str[0:4])
            month = int(date_str[5:7])
            day = int(date_str[8:10])
            return date(year, month, day)
        else:
            return None

    @staticmethod
    def _parse_counter_account(element: Element) -> Optional[str]:
        account_number = element.findtext('column_2', None)
        bank_id = element.findtext('column_3', None)
        if account_number and bank_id:
            return account_number + '/' + bank_id
        else:
            return None
