#
# Copyright (C) 2020  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.

"""A module providing data classes to model bank statements."""
from datetime import date
from typing import Iterator, List

from moneyed import Money


class BankStatement:
    """Data class to model bank statement."""

    def __init__(self, account_number: str):
        self.account_number = account_number
        self.payments = []  # type: List[Payment]

    def __iter__(self) -> Iterator['Payment']:
        return iter(self.payments)

    def add_payment(self, payment: 'Payment') -> None:
        """Add payment to the statement."""
        self.payments.append(payment)

    def validate(self) -> None:
        """Validate that the account_number is set and payments are valid. Raise ValueError if validation fails."""
        if not self.account_number:
            raise ValueError('Account number has to be set.')
        for payment in self.payments:
            payment.validate()


class Payment:
    """Data class to model a transaction in bank statement."""

    def __init__(self, identifier: str = None, transaction_date: date = None, counter_account: str = None,
                 name: str = None, amount: Money = None, description: str = None, constant_symbol: str = None,
                 variable_symbol: str = None, specific_symbol: str = None):
        self.identifier = identifier
        self.transaction_date = transaction_date
        self.counter_account = counter_account
        self.name = name
        self.amount = amount
        self.description = description
        self.constant_symbol = constant_symbol
        self.variable_symbol = variable_symbol
        self.specific_symbol = specific_symbol

    def validate(self) -> None:
        """Validate the required fileds. Raise ValueError is if validation fails."""
        if not self.identifier:
            raise ValueError('Payment identifier has to be set.')

        if self.amount is None:
            raise ValueError('Amount has to be set.')
