#
# Copyright (C) 2020  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.
from datetime import date
from unittest import TestCase

import responses

from teller.downloaders import BankStatementDownloader, FioStatementDownloader, TellerDownloadError


class TestBankStatementDownloader(TestCase):
    def test_is_abstract(self):
        with self.assertRaisesRegex(TypeError, "Can't instantiate abstract class"):
            BankStatementDownloader('', 1)

    def test_enforce_https(self):
        class DummyStatementDownloader(BankStatementDownloader):
            def get_statement(self, start_date, end_date):
                pass  # pragma: no cover

        with self.assertWarnsRegex(UserWarning, 'Sensitive data is transferred'):
            DummyStatementDownloader('http://bank.test', 1)


class TestFioStatementDownloader(TestCase):
    url = 'https://bank.test'
    password = 'letmein'
    start_date = date(2020, 9, 3)
    end_date = date(2020, 10, 2)

    def test_compose_url(self):
        downloader = FioStatementDownloader(self.url, self.password)

        result = downloader._compose_url(self.start_date, self.end_date)

        full_url = '{}/ib_api/rest/periods/{}/2020-09-03/2020-10-02/transactions.xml'.format(self.url, self.password)
        self.assertEqual(result, full_url)

    def test_get_statement(self):
        downloader = FioStatementDownloader(self.url, self.password)
        statement = 'I am a fish.'

        full_url = '{}/ib_api/rest/periods/{}/2020-09-03/2020-10-02/transactions.xml'.format(self.url, self.password)
        with responses.RequestsMock() as rsps:
            rsps.add(responses.GET, full_url, body=statement, status=200, content_type='text/plain')
            result = downloader.get_statement(self.start_date, self.end_date)

        self.assertEqual(result, statement)

    def test_get_statement_not_found(self):
        downloader = FioStatementDownloader(self.url, self.password)

        full_url = '{}/ib_api/rest/periods/{}/2020-09-03/2020-10-02/transactions.xml'.format(self.url, self.password)
        with responses.RequestsMock() as rsps:
            rsps.add(responses.GET, full_url, body='Not found.', status=404, content_type='text/plain')
            with self.assertRaisesRegex(TellerDownloadError, 'Statement could not be retrieved'):
                downloader.get_statement(self.start_date, self.end_date)
