#
# Copyright (C) 2020  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.
from datetime import date
from io import StringIO
from pathlib import Path
from unittest import TestCase

from moneyed import Money

from teller.parsers import BankStatementParser, FioParser


class TestBankStatementParser(TestCase):
    def test_parse_raises_exception(self):
        with self.assertRaisesRegex(NotImplementedError, 'This is abstract base class'):
            BankStatementParser.parse_string('')

        with self.assertRaisesRegex(NotImplementedError, 'This is abstract base class'):
            BankStatementParser.parse_file('/tmp/a.txt')


class TestFioParser(TestCase):

    path = Path('teller/tests/data/example_fio.xml')

    def test_parse_header_path(self):
        statement = FioParser.parse_file(self.path)
        self.assertEqual(statement.account_number, '1234567890/2010')

    def test_parse_header_file(self):
        with self.path.open() as f:
            statement = FioParser.parse_file(f)
        self.assertEqual(statement.account_number, '1234567890/2010')

    def test_parse_header_str(self):
        xml = '<AccountStatement><Info>'
        xml += '<accountId>1234567890</accountId><bankId>2010</bankId>'
        xml += '</Info>'
        xml += '<TransactionList></TransactionList>'
        xml += '</AccountStatement>'

        statement = FioParser.parse_string(xml)
        self.assertEqual(statement.account_number, '1234567890/2010')

    def test_parse_header_stringio(self):
        xml = '<AccountStatement><Info>'
        xml += '<accountId>1234567890</accountId><bankId>2010</bankId>'
        xml += '</Info>'
        xml += '<TransactionList></TransactionList>'
        xml += '</AccountStatement>'

        statement = FioParser.parse_file(StringIO(xml))
        self.assertEqual(statement.account_number, '1234567890/2010')

    def test_missing_header(self):
        xml = '<AccountStatement></AccountStatement>'
        with self.assertRaisesRegex(ValueError, 'Statement is missing header\\.'):
            FioParser.parse_string(xml)

    def test_missing_account_number(self):
        xml = '<AccountStatement><Info></Info></AccountStatement>'
        with self.assertRaisesRegex(ValueError, 'Could not parse account number\\.'):
            FioParser.parse_string(xml)

    def test_parse_payment(self):
        statement = FioParser.parse_file(self.path)
        payment = statement.payments[1]

        self.assertEqual(payment.identifier, '5800012966')
        self.assertEqual(payment.transaction_date, date(2012, 12, 21))
        self.assertEqual(payment.counter_account, '3456789012/2010')
        self.assertEqual(payment.name, 'Jan Novak')
        self.assertEqual(payment.amount, Money(3000, 'CZK'))
        self.assertEqual(payment.description, 'Vesele vanoce <> & Novy rok')
        self.assertEqual(payment.constant_symbol, '0234')
        self.assertEqual(payment.variable_symbol, '222222')
        self.assertEqual(payment.specific_symbol, '333333')

    def test_parse_multiple_payments(self):
        statement = FioParser.parse_file(self.path)
        self.assertEqual(len(statement.payments), 2)
        self.assertEqual(sum(p.amount for p in statement.payments), Money(4000, 'CZK'))

    def test_no_transactions(self):
        xml = '<AccountStatement><Info>'
        xml += '<accountId>1234567890</accountId><bankId>2010</bankId>'
        xml += '</Info>'
        xml += '<TransactionList></TransactionList>'
        xml += '</AccountStatement>'

        statement = FioParser.parse_string(xml)
        self.assertEqual(len(statement.payments), 0)

    def test_empty_transaction(self):
        xml = '<AccountStatement><Info>'
        xml += '<accountId>1234567890</accountId><bankId>2010</bankId>'
        xml += '</Info>'
        xml += '<TransactionList><Transaction></Transaction></TransactionList>'
        xml += '</AccountStatement>'

        statement = FioParser.parse_string(xml)
        self.assertEqual(statement.payments[0].amount, None)
        self.assertEqual(statement.payments[0].counter_account, None)
        self.assertEqual(statement.payments[0].transaction_date, None)

    def test_invalid_date_raises_exception(self):
        xml = '<AccountStatement><Info>'
        xml += '<accountId>1234567890</accountId><bankId>2010</bankId>'
        xml += '</Info>'
        xml += '<TransactionList><Transaction><column_0>not a date</column_0></Transaction></TransactionList>'
        xml += '</AccountStatement>'

        with self.assertRaisesRegex(ValueError, 'invalid literal for int'):
            FioParser.parse_string(xml)
