#
# Copyright (C) 2020  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.

from datetime import date
from decimal import Decimal
from unittest import TestCase

from moneyed import Money

from teller.statement import BankStatement, Payment


class TestBankStatement(TestCase):
    def test_validate(self):
        statement = BankStatement('123456/7890')
        statement.add_payment(Payment(identifier='abc', amount=Decimal(1)))
        statement.validate()

    def test_does_not_validate_missing_account(self):
        with self.assertRaisesRegex(ValueError, 'Account number has to be set\\.'):
            BankStatement('').validate()

    def test_does_not_validate_invalid_payments(self):
        statement = BankStatement('123456/7890')
        statement.add_payment(Payment())
        with self.assertRaisesRegex(ValueError, '.*has to be set\\.$'):
            statement.validate()


class TestPayment(TestCase):
    parameters = {'identifier': 'abc',
                  'transaction_date': date(2020, 1, 15),
                  'counter_account': '987654/3210',
                  'name': 'John Doe',
                  'amount': Money(3.14, 'GBP'),
                  'description': 'Have a nice day.',
                  'constant_symbol': '123',
                  'variable_symbol': '456',
                  'specific_symbol': '789'}

    def test_parameters(self):
        payment = Payment(**self.parameters)

        self.assertEqual(payment.identifier, 'abc')
        self.assertEqual(payment.transaction_date, date(2020, 1, 15))
        self.assertEqual(payment.counter_account, '987654/3210')
        self.assertEqual(payment.name, 'John Doe')
        self.assertEqual(payment.amount, Money(3.14, 'GBP'))
        self.assertEqual(payment.description, 'Have a nice day.')
        self.assertEqual(payment.constant_symbol, '123')
        self.assertEqual(payment.variable_symbol, '456')
        self.assertEqual(payment.specific_symbol, '789')

    def test_validate(self):
        Payment(**self.parameters).validate()

    def test_does_not_validate_missign_identifier(self):
        payment = Payment(**self.parameters)
        payment.identifier = ''
        with self.assertRaisesRegex(ValueError, 'Payment identifier has to be set\\.'):
            payment.validate()

    def test_does_not_validate_missign_amount(self):
        payment = Payment(**self.parameters)
        payment.amount = None
        with self.assertRaisesRegex(ValueError, 'Amount has to be set\\.'):
            payment.validate()
